@@ -230,18 +230,26 @@ sub where
 
 	my @where_class_id;
 
-	if ($classes->{$class}{stateless})
+	if (0 and $classes->{$class}{stateless})
 	{
 		my @class_ids;
 
-		push @class_ids, $storage->class_id($class) unless $classes->{$class}{abstract};
+		push @class_ids, $storage->class_id($class)
+		    unless $classes->{$class}{abstract};
 
-		$schema->for_each_spec($class,
-							   sub { my $spec = shift; push @class_ids, $storage->class_id($spec) unless $classes->{$spec}{abstract} } );
+		$schema->for_each_spec
+		    ($class,
+		     sub {
+			 my $spec = shift;
+			 push @class_ids, $storage->class_id($spec)
+			     unless $classes->{$spec}{abstract};
+		     } );
 
-		@where_class_id = "t$root.$storage->{class_col} IN (" . join(', ', $storage->_kind_class_ids($class) ) . ')';
 	}
 
+	@where_class_id = "t$root.$storage->{class_col} IN ("
+	    . join(', ', $storage->_kind_class_ids($class) ) . ')';
+
 	my $id = $schema->{sql}{id_col};
 	return (@where_class_id, map { "t@{$_}[1].$id = t$root.$id" } @$tables[1..$#$tables]);
 }
